##########################################################################
#
# __init__.py:  defines this directory as the 'tests' package.
#
#  Debian Partial Mirror is a tool to make complete and partial Debian mirrors.
#  See http://projetos.ossystems.com.br/debpartial-mirror/ for more information.
#
# ====================================================================
# Copyright (c) 2002-2005 O.S. Systems.  All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution.
#
#########################################################################
# Author: Otavio Salvador <otavio@ossystems.com.br>

import unittest

import TestConfig
import TestDownload
import TestEndToEnd

def suite():
	suite = unittest.TestSuite()
	suite.addTest(TestConfig.suite())
	suite.addTest(TestDownload.suite())
	suite.addTest(TestEndToEnd.suite())
	return suite


if __name__ == '__main__':
	unittest.main(defaultTest='suite')
