#!/usr/bin/python
"""
Tool to build partial debian mirrors (and more)
"""
# debpartial-mirror - partial debian mirror package tool
# (c) 2006 Otavio Salvador <otavio@debian.org>
# (c) 2006 Marco Presi <zufus@debian.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# -------
# Imports
# -------


import gtk

from debpartial_mirror_ui import Callbacks
from debpartial_mirror_ui import GDebPartialMirror


if __name__ == "__main__":
    ui = GDebPartialMirror.GDebPartialMirror("/etc/debpartial-mirror.conf")
    gtk.main()

