from os import sys

sys.path.append ('../')

from gtk import Window, main, main_quit
from ConfigEditor import *
from debpartial_mirror import Config, Backend

cnf = Config.Config('/etc/debpartial-mirror.conf')

mirrors = []
merges = []

for b in cnf.get_backends()[0]:
    mirrors.append(Backend.MirrorBackend(b.section,cnf))

for b in cnf.get_backends()[1]:
    merges.append(Backend.MirrorBackend(b.section,cnf))


w = Window()
bb = ConfigEditor (cnf, debug = True)
w.add(bb)

w.show_all()
w.connect('destroy', main_quit)
main()
