#!/usr/bin/python
import unittest
import sys

sys.path.append('lib')

import __init__

if not unittest.TextTestRunner(verbosity=2).\
                               run(__init__.suite()).wasSuccessful():
    sys.exit(1)
