##########################################################################
#
# __init__.py:  defines this directory as 'debpartial_mirror' package.
#
#  debpartial_mirror is an application to make easier to build full or
#  partial mirrors from Debian or compatible archives.
#  See http://partial-mirror.alioth.debian.org/ for more information.
#    
# ====================================================================
# Copyright (c) 2002-2005 OS Systems.  All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution.
#
#########################################################################
# Author: Otavio Salvador <otavio@ossystems.com.br>

